/*
	Phoenix Suite 2
	Author: Jean-Baptiste Wautier
	Copyright 2006-2009 (c)
*/
﻿namespace Phoenix.Log.Writer
{
    using System;
    using System.Collections.Generic;
    using System.Text;

    /// <summary>
    /// Logs into the console
    /// </summary>
    public class ConsoleAppender : RootAppender, IAppender
    {
        #region Fields

        private static readonly ConsoleColor BackgroundColor = Console.BackgroundColor;
        private static readonly ConsoleColor ForegroundColor = Console.ForegroundColor;

        private List<string> message = new List<string>();

        #endregion Fields

        #region Constructors

        /// <summary>
        /// Initializes assembly new instance of the <see cref="ConsoleAppender"/> class.
        /// </summary>
        public ConsoleAppender()
        {
        }

        #endregion Constructors

        #region Methods

        /// <summary>
        /// Inserts the specified error baseLog.
        /// </summary>
        /// <param name="logRow">The error baseLog.</param>
        protected override void ProcessInsert(IRow logRow)
        {
            if (logRow == null)
                throw new ArgumentException("The argument shouldn't be null", "logRow");

            if (base.LogLevel < (int)logRow.Stamp)
                return;

            if (logRow.Deepness == 0)
            {
                SetColor(logRow.Stamp);
                Console.WriteLine("[{0}]", logRow.Stamp.ToString());
                Console.WriteLine("[{0}]", logRow.InsertDate.ToString());
                Console.WriteLine("{0}", logRow.Title);
                Console.WriteLine();

                Console.WriteLine("Message:");
                Console.WriteLine("--------");
                this.message.Add(logRow.Message + Environment.NewLine);
                string msg = string.Empty;
                string separator = "-----------------------------------------------------" + Environment.NewLine;
                for (int i = this.message.Count - 1; i >= 0; i--)
                {
                    msg += this.message[i] + separator;
                }
                Console.WriteLine(msg + logRow.ExceptionMessage
                    + Environment.NewLine + separator);
                this.message.Clear();
                Console.WriteLine();

                Console.WriteLine("Stack trace:");
                Console.WriteLine("------------");
                Console.WriteLine(logRow.StackTrace);
                Console.WriteLine();

                Console.WriteLine("Source:");
                Console.WriteLine("-------");
                Console.WriteLine(logRow.Source);
                Console.WriteLine("=====================================================");
                RestoreConsoleColor();
            }
            else
            {
                this.message.Add(logRow.Message + Environment.NewLine);
            }
        }

        private static void RestoreConsoleColor()
        {
            Console.ForegroundColor = ForegroundColor;
            Console.BackgroundColor = BackgroundColor;
        }

        private static void SetColor(Log.Stamp stamp)
        {
            switch (stamp)
            {
                case Log.Stamp.Fatal:
                    Console.BackgroundColor = ConsoleColor.Red;
                    Console.ForegroundColor = ConsoleColor.Black;
                    break;
                case Log.Stamp.Error:
                    Console.BackgroundColor = ConsoleColor.Black;
                    Console.ForegroundColor = ConsoleColor.Red;
                    break;
                case Log.Stamp.Warning:
                    Console.BackgroundColor = ConsoleColor.Black;
                    Console.ForegroundColor = ConsoleColor.Yellow;
                    break;
                case Log.Stamp.Info:
                    Console.BackgroundColor = ConsoleColor.Black;
                    Console.ForegroundColor = ConsoleColor.White;
                    break;
                case Log.Stamp.Flag:
                    Console.BackgroundColor = ConsoleColor.Black;
                    Console.ForegroundColor = ConsoleColor.Cyan;
                    break;
                case Log.Stamp.Debug:
                    Console.BackgroundColor = ConsoleColor.Black;
                    Console.ForegroundColor = ConsoleColor.Green;
                    break;
                default:
                    Console.BackgroundColor = ConsoleColor.Black;
                    Console.ForegroundColor = ConsoleColor.Gray;
                    break;
            }
        }

        #endregion Methods
    }
}